/****************************************************************************

 This file is part of GLC-Player.
 Copyright (C) 2007-2008 Laurent Ribon (laumaya@users.sourceforge.net)
 Version 2.0.0, packaged on March 2009.

 http://www.glc-player.net

 GLC-Player is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 GLC-Player is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with GLC-Player; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

*****************************************************************************/

#include "MultiShotsOpenglView.h"
#include <GLC_Exception>
#include <GLC_State>

MultiShotsOpenglView::MultiShotsOpenglView(QWidget * pParent, QGLWidget* pShareWidget)
: QGLWidget(pParent, pShareWidget)
, m_GlView(this)
, m_World()
, m_Light()
, m_MotionTimer()
, m_RotationVector()
, m_Camera()
, m_Angle()
, m_pOpenglView(dynamic_cast<OpenglView*>(pShareWidget))
, m_MotionStep(1)
, m_MotionLength(1000)
{
	m_Light.setPosition(1.0, 1.0, 1.0);

	// Set backroundImage
	m_GlView.loadBackGroundImage(":images/default_background.png");

	// Signal and slot connection
	connect(&m_MotionTimer, SIGNAL(timeout()), this, SLOT(rotateView()));

}

MultiShotsOpenglView::~MultiShotsOpenglView()
{

}

//////////////////////////////////////////////////////////////////////
// Public Set Functions
//////////////////////////////////////////////////////////////////////
// Init the view
void MultiShotsOpenglView::initView()
{

	m_World= m_pOpenglView->getWorld();
	m_Camera= m_pOpenglView->getCamera();
	double xUp= fabs(m_Camera.getVectUp().X());
	double yUp= fabs(m_Camera.getVectUp().Y());
	double zUp= fabs(m_Camera.getVectUp().Z());
	if ((xUp > yUp) and (xUp > zUp))
	{
		m_RotationVector= glc::X_AXIS;
	}
	else if ((yUp > xUp) and (yUp > zUp))
	{
		m_RotationVector= glc::Y_AXIS;
	}
	else
	{
		m_RotationVector= glc::Z_AXIS;
	}
	m_Angle= m_pOpenglView->getViewAngle();

}

// Update the motion Step
void MultiShotsOpenglView::updateMotionStepAndLength(int step, int motionLength)
{
	m_MotionStep= step;
	// Second -> millisecond
	m_MotionLength= motionLength * 1000;

	m_MotionTimer.start(m_MotionLength / step);
}

// Update Gravity Axis
void MultiShotsOpenglView::updateGravityAxis(const GLC_Vector4d& newAxis)
{
	if (newAxis == -m_RotationVector)
	{
		m_RotationVector= newAxis;
	}
	else if (newAxis != m_RotationVector)
	{
		m_RotationVector= newAxis;
		(*m_GlView.m_pViewCam)= m_Camera;
	}
}

//////////////////////////////////////////////////////////////////////
// Portected services Functions
//////////////////////////////////////////////////////////////////////

// Initialize OpenGL window
void MultiShotsOpenglView::initializeGL()
{
	m_GlView.initGl();

	glEnable(GL_NORMALIZE);

	(*m_GlView.m_pViewCam)= m_Camera;
	m_GlView.setFov(m_Angle);

}

// paint in the opengl widget
void MultiShotsOpenglView::paintGL()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	m_GlView.setDistMinAndMax(m_World.collection()->boundingBox());

	// Enable and execute lighting
	m_Light.enable();
	try
	{
		m_Light.glExecute();
		m_GlView.glExecuteCam();
		// Test if there is a global shader
		const GLuint globalShaderId= m_pOpenglView->globalShaderId();
		if (0 != globalShaderId) GLC_Shader::use(globalShaderId);


		// Display non transparent normal object
		m_World.glExecute(0, false);
		// Display non transparent instance of the shaders group
		if (GLC_State::glslUsed())
		{
			m_World.glExecuteShaderGroup(false);
		}

		// Display transparent normal object
		m_World.glExecute(0, true);
		// Display transparent instance of the shaders group
		if (GLC_State::glslUsed())
		{
			m_World.glExecuteShaderGroup(true);
		}
		// Display Selected Objects
		const int numberOfSlectedNode= m_World.collection()->numberOfSelectedNode();
		if ((numberOfSlectedNode > 0) and GLC_State::selectionShaderUsed())
		{
			if (numberOfSlectedNode != m_World.collection()->numberOfDrawableObjects())
			{
				//Draw the selection with Zbuffer
				m_World.glExecute(1);
			}
			// Draw the selection transparent
			glPushAttrib(GL_ENABLE_BIT | GL_DEPTH_BUFFER_BIT);
	        glEnable(GL_CULL_FACE);
	        glEnable(GL_BLEND);
	        glDepthFunc(GL_ALWAYS);
	        glBlendFunc(GL_SRC_ALPHA,GL_ONE);

	        m_World.glExecute(1);

    	    // Restore attributtes
	        glPopAttrib();
		}
		else if (numberOfSlectedNode > 0)
		{
			m_World.glExecute(1);
		}
		// Test if there is a global shader
		if (0 != globalShaderId) GLC_Shader::unuse();

	}
	catch (GLC_Exception &e)
	{
		qDebug() << e.what();
	}

}

// The widget as been resized
void MultiShotsOpenglView::resizeGL(int width, int height)
{
	m_GlView.setWinGLSize(width, height);
}

//////////////////////////////////////////////////////////////////////
// Private slots Functions
//////////////////////////////////////////////////////////////////////
// Rotate the view
void MultiShotsOpenglView::rotateView()
{
	GLC_Matrix4x4 RotationMatrix(m_RotationVector, 2.0 * glc::PI / static_cast<double>(m_MotionStep));
	m_GlView.m_pViewCam->translate(- m_Camera.getTarget());
	m_GlView.m_pViewCam->move(RotationMatrix);
	m_GlView.m_pViewCam->translate(m_Camera.getTarget());
	updateGL();
}
